@@ -74,6 +74,25 @@ public :
     #endif //MEDIAINFO_DEMUX
     Ztring  File_Name_WithoutDemux;
     bool   PTS_DTS_Needed;
+    enum ts_type
+    {
+        TS_NONE=0,
+        TS_PTS=1,
+        TS_DTS=2,
+        TS_ALL=TS_PTS|TS_DTS,
+    };
+    void   TS_Clear(ts_type Type=TS_ALL);
+    void   TS_Set(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Set(File__Analyze* Parser, ts_type Type=TS_ALL);
+    void   TS_Add(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Ajust(int64s Ticks);
+    int64s Frequency_c; //Frequency of the timestamp of the container (e.g. 90000 for MPEG-PS)
+    int64s Frequency_b; //Frequency of the timestamp of the bitstream (e.g. 48000 for AC-3)
+    #if MEDIAINFO_ADVANCED2
+    static const int64s NoTs=0x8000000000000000LL;
+    int64s PTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    int64s DTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    #endif //MEDIAINFO_ADVANCED2
     struct frame_info
     {
         int64u Buffer_Offset_End;
@@ -81,6 +100,11 @@ public :
         int64u PTS; //In nanoseconds
         int64u DTS; //In nanoseconds
         int64u DUR; //In nanoseconds
+        #if MEDIAINFO_ADVANCED2
+        int64s PTSc; //In 1/Frequency_c
+        int64s DTSc; //In 1/Frequency_c
+        int64u Frame_Count_AfterLastTimeStamp;
+        #endif //MEDIAINFO_ADVANCED2
 
         frame_info()
         {
@@ -89,6 +113,11 @@ public :
             PTS=(int64u)-1;
             DTS=(int64u)-1;
             DUR=(int64u)-1;
+            #if MEDIAINFO_ADVANCED2
+            PTSc=NoTs;
+            DTSc=NoTs;
+            Frame_Count_AfterLastTimeStamp=0;
+            #endif //MEDIAINFO_ADVANCED2
         }
     };
     frame_info FrameInfo;
@@ -118,6 +147,20 @@ public :
         };
         servicedescriptors* ServiceDescriptors;
     #endif
+    #if defined(MEDIAINFO_TELETEXT_YES)
+        struct teletext
+        {
+            std::map<std::string, Ztring>           Infos;
+            stream_t                                StreamKind;
+            size_t                                  StreamPos;
+
+            teletext()
+                : StreamKind(Stream_Max)
+                , StreamPos((size_t)-1)
+            {}
+        };
+        std::map<int16u, teletext>*                 Teletexts; //Key is teletext_magazine_number
+    #endif
 
     //Out
     int64u PTS_Begin;                  //In nanoseconds
@@ -275,6 +318,7 @@ protected :
     #define Element_Info3(_A,_B,_C)
     #define Element_Info1C(_CONDITION,_A)
     #define Element_Info_From_Milliseconds(_A)
+    #define Element_Parser(_A)
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -986,9 +1030,9 @@ public :
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int32s         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int64u         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, int64s         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
-    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float32        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, AfterComma), Replace);}
-    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float64        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, AfterComma), Replace);}
-    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float80        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, AfterComma), Replace);}
+    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float32        Value, int8u AfterComma=3, bool Replace=false);
+    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float64        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, (float32)Value, AfterComma, Replace);}
+    inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, float80        Value, int8u AfterComma=3, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, (float32)Value, AfterComma, Replace);}
     #ifdef SIZE_T_IS_LONG
     inline void Fill (stream_t StreamKind, size_t StreamPos, size_t Parameter, size_t         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     #endif //SIZE_T_IS_LONG
@@ -1281,6 +1325,19 @@ public :
     //Configuration
     bool MustSynchronize;
     bool CA_system_ID_MustSkipSlices;
+    bool FillAllMergedStreams;
+    class streamidentity
+    {
+    public:
+        stream_t StreamKind;
+        size_t   StreamPos;
+
+        streamidentity(stream_t StreamKind_, size_t StreamPos_)
+            : StreamKind(StreamKind_)
+            , StreamPos(StreamPos_)
+        {}
+    };
+    std::vector<streamidentity> MergedStreams_Last;
 
     //Demux
     enum contenttype
@@ -1315,11 +1372,11 @@ public :
         size_t      AES_Decrypted_Size;
     #endif //MEDIAINFO_AES
 
-    //MD5
-    #if MEDIAINFO_MD5
-        struct MD5Context*  MD5;
-        int64u              Md5_ParseUpTo;
-    #endif //MEDIAINFO_MD5
+    //Hash
+    #if MEDIAINFO_HASH
+        HashWrapper*        Hash;
+        int64u              Hash_ParseUpTo;
+    #endif //MEDIAINFO_HASH
 
     #if MEDIAINFO_SEEK
     private:
